/* Program to allow comparison of tag sequence probabilities

   26-01-93 Created
   16 Jan 2004 Cleaned up for open source release.
    
   Copyright (C) David Elworthy 2004.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   For details, see the file LICENCE, or visit http://www.gnu.org/copyleft/gpl.html

   Usage:
     cmptran trans
   
  Takes lines consisting of three tags and works out the probability of the
  sequence, and the ratio of this one to the last (on the second of each
  pair).
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"
#include "trans.h"
#include "map.h"

/*-----------------------------------------------------------------------------
    main

    Entry point.
-----------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
#define MaxLine (1000)
    Trans trans;
    char buffer[MaxLine];
    int i = 0;
    Score s = 0;

    /* Clear data structures */
    InitTrans(trans)
    InitOptions;

    /* Verify command line */
    if (argc <= 1)
        error_exit("Usage: cmptran trans\n");

    read_mapping("tags.map");
    read_named_trans(argv[1], &trans);
    adjust_trans(&trans, NULL);

    /* Read from stdin until end of file */
    while (fgets(buffer, MaxLine-1, stdin) != NULL)
    {
        uchar *tag1, *tag2, *tag3;
        Tag  t1, t2, t3;
        Score t;

        tag1 = strtok(buffer, " \t:\n");
        tag2 = strtok(NULL,   " \t:\n");
        tag3 = strtok(NULL,   " \t:\n");
        t1 = map_tag(tag1);
        t2 = map_tag(tag2);
        t3 = map_tag(tag3);

        t = Trans_(trans.trans,t1,t2,trans.size)
            * Trans_(trans.trans,t2,t3,trans.size);
        printf("%s : %s : %s = %g", unmap_tag(t1), unmap_tag(t2), unmap_tag(t3), t);
        if (i == 0) s = t;
        else printf(" %g", t/s);
        printf("\n");
        i = 1 - i;
    }

    return 0;
}
