/* Statistical labeller: sorted list handler.

   2-1-92 Created
   4-2-92 Key types changed.
   16 Jan 2004 Cleaned up for open source release.
    
   Copyright (C) David Elworthy 2004.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   For details, see the file LICENCE, or visit http://www.gnu.org/copyleft/gpl.html

   See list.c for the sort of structures that must be used with the list
   handler.
*/

/* Type for a key - cast the actual key to this. In a list structure, the
second field has type Key. The comparison function is called with a pointer to
this field and with the object passed as "key" to the list_ functions. The
first argument is always the given key, the second the one from the list. */
typedef int *Key;

/* Type for a comparison function on list members. */
typedef int (List_Cmp)(Key *, Key *);

/* Find a member in a list */
extern void *list_search(void *head, Key *key, List_Cmp *compar, void **next,
    BOOL *match);

/* Add a member to a list */
extern void *list_add(void **prev, void *next, int size, char *text);

/* Find and add if necessary */
extern void *list_search_and_add(void **head, Key *key, List_Cmp *compar,
    int size, char *text, BOOL *match);

/* Find and add, giving error on duplicate. Return NULL on error */
extern void *list_search_and_check(void **head, Key *key, List_Cmp *compar,
    int size, char *text, char *text1);
